import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'package:build/build.dart';
import 'package:logging/logging.dart';
import 'package:package_resolver/package_resolver.dart';
import 'package:path/path.dart';
import 'package:sass/sass.dart';

class SassBuilder implements Builder {

  Logger _log = new Logger('SassBuilder');

  Set<AssetId> _mainInputs;

  Set<AssetId> get mainInputs {
    if (_mainInputs == null) {
      var sassBuildFile = new File(join(Directory.current.path, '.dart_tool', 'sass_build_main_inputs.json'));
      _log.info('reading main_inputs file: ${sassBuildFile}');
      if (sassBuildFile.existsSync()) {
        var mainInputsJson = sassBuildFile.readAsStringSync();
        mainInputsJson = mainInputsJson.isEmpty ? '[]' : mainInputsJson;
        List mainInputsList = JSON.decode(mainInputsJson);
        mainInputsList = mainInputsList == null || mainInputsList.isEmpty ? [] : mainInputsList;
        _mainInputs = new Set();
        var rewriteInputs = false;
        for (var mainInputAssetIdName in mainInputsList) {
          var mainInputAssetId = new AssetId.parse(mainInputAssetIdName);
          if (new File(mainInputAssetId.path).existsSync()) {
            _log.info('adding mainInputFile: ${mainInputAssetId}');
            _mainInputs.add(mainInputAssetId);
          } else {
            rewriteInputs = true;
          }
        }
        if (rewriteInputs) {
          _log.info('rewriting inputs');
          _saveMainInputs();
        }
      } else {
        _mainInputs = new Set();
      }
    }
    return _mainInputs;
  }

  _saveMainInputs() {
    var sassBuildFile = new File(join(Directory.current.path, '.dart_tool', 'sass_build_main_inputs.json'));
    if (!sassBuildFile.existsSync()) sassBuildFile.createSync(recursive: true);

    _log.info('writing mainInputs: $_mainInputs');
    sassBuildFile.writeAsStringSync(JSON.encode(_mainInputs.map((mi) => mi.toString()).toList()));
  }

  @override
  Future build(BuildStep buildStep) async {
    var inputId = buildStep.inputId;
    _log.info('processing changed file: ${inputId}');

    if (!basename(inputId.path).startsWith('_')) {
      if (!mainInputs.contains(inputId)) {
        mainInputs.add(inputId);
        _saveMainInputs();
      }
    }
    _log.info('using mainInputs: $mainInputs');

    if (mainInputs.isNotEmpty) {
      var mainInputsToRemove = new Set();
      for (var mainInput in mainInputs) {
        _log.info('parsing: ${mainInput}');
        if (!new File(mainInput.path).existsSync()) {
          _log.info('adding mainInput to remove: $mainInput');
          mainInputsToRemove.add(mainInput);
          continue;
        }
        var css = await compile(mainInput.path, packageResolver: await PackageResolver.current.asSync);
//        buildStep.writeAsString(_changeExtension(mainInput), css);
        var file = new File(mainInput
            .changeExtension('.css')
            .path);
        file.createSync(recursive: true);
        file.writeAsString(css);
      }

      _log.info('removing mainInputs non existing');
      mainInputs.removeAll(mainInputsToRemove);
    }
  }

  @override
  Map<String, List<String>> get buildExtensions =>
      {'': const ['.css']}; // since we are writing files without buildStep, we need to have a fake extension
}
